<?php

namespace FiloBlu\Refilo\Model\Indexer\Processor\Type;

class Simple extends AbstractType
{

    /**
     * @return \stdClass
     */
    public function process()
    {
        $this->_item = new \stdClass();

        $this->processBasedata();
        $this->processAssets();
        $this->processAttributes();

        $this->processCategories();
//        $this->processPrices();
//        $this->processInventory();

        return $this->_item;
    }

    protected function processBasedata()
    {
        $this->_item->_id = (int) $this->_product->getId();
        $this->_item->id = (int) $this->_product->getId();
        $this->_item->sku = $this->_product->getSku();
        $this->_item->name = $this->_product->getName();
        $this->_item->docType = 'product';

        $this->_item->type = $this->_product->getData('type_id');
        $this->_item->created_at = $this->_product->getData('created_at');
        $this->_item->updated_at = $this->_product->getData('updated_at');

        $this->_item->weight = $this->_product->getData('weight');
        $this->_item->visible = ((int)$this->_product->getData('visibility') == 1) ? "0" : "1";
    }

    protected function processAssets()
    {
        $assets = $this->_product->getData('media_gallery');

        $this->_item->assets = new \stdClass();

        if (count($assets['images'])) {
            foreach ($assets['images'] as $key=>$imageobj) {
                $this->_item->assets->images[] = ['label' => $imageobj['label'], 'file' => ($imageobj['file']) ? '/catalog/product' . $imageobj['file'] : null];
            }
        }

        $this->_item->assets->placeholders = new \stdClass();
        $this->_item->assets->placeholders->image = ($this->_product->getData('image')) ? '/catalog/product' . $this->_product->getData('image') : null;
        $this->_item->assets->placeholders->small_image =  ($this->_product->getData('small_image')) ? '/catalog/product' . $this->_product->getData('small_image') : null;
        $this->_item->assets->placeholders->thumbnail = ($this->_product->getData('thumbnail')) ? '/catalog/product' . $this->_product->getData('thumbnail') : null;
//        $this->_item->assets->placeholders->swatch_image = ($this->_product->getData('swatch_image')) ? '/catalog/product' . $this->_product->getData('swatch_image') : null;
    }

    protected function includeInIndex($attribute)
    {
        if (
            $attribute->getAttributeCode() == 'price' ||
            !isset($this->_product_attributes[$attribute->getAttributeCode()]) ||
            !$attribute->getIsVisible() ||
            $this->_product_attributes[$attribute->getAttributeCode()]['frontend_input'] === 'multiselect') {
            return false;
        }

        if (
            (int) !$this->_product_attributes[$attribute->getAttributeCode()]['is_filterable'] &&
            (int) !$this->_product_attributes[$attribute->getAttributeCode()]['used_for_sort_by'] &&
            (int) !$this->_product_attributes[$attribute->getAttributeCode()]['is_searchable'] &&
            (int) !$this->_product_attributes[$attribute->getAttributeCode()]['is_visible_on_front'] &&
            (int) !$this->_product_attributes[$attribute->getAttributeCode()]['is_filterable_in_search'] &&
            (int) !$this->_product_attributes[$attribute->getAttributeCode()]['used_in_product_listing'] &&
            (int) !$this->_product_attributes[$attribute->getAttributeCode()]['is_visible_in_advanced_search']

        ) {
            return false;
        }

        if (in_array($attribute->getAttributeCode(), $this->excludedAttributes)) {
            return false;
        }

        return true;
    }

    protected function buildBaseAttributeData($attribute)
    {
        $attribute_code = $attribute->getAttributeCode();

        $this->_item->attributes->{$attribute_code} = new \stdClass();

        $this->_item->attributes->{$attribute_code}->title = $this->_product_attributes[$attribute->getAttributeCode()]['frontend_label'];
//        $this->_item->attributes->{$attribute_code}->filterable = (int) $this->_product_attributes[$attribute->getAttributeCode()]['is_filterable'];
//        $this->_item->attributes->{$attribute_code}->sortable = (int) $this->_product_attributes[$attribute->getAttributeCode()]['used_for_sort_by'];
        $this->_item->attributes->{$attribute_code}->code = $attribute_code;
        $this->_item->attributes->{$attribute_code}->type = $this->_product_attributes[$attribute->getAttributeCode()]['frontend_input'];
    }

    protected function buildInputAttributeData($attribute)
    {
        $attribute_code = $attribute->getAttributeCode();
        $attribute_id = $attribute->getAttributeId();

        $attribute_key = ($this->_product->getData($attribute->getAttributeCode()) !== null) ? ($this->_product->getData($attribute->getAttributeCode())) : null;
        $this->_item->attributes->{$attribute_code}->key = $attribute_key;

//            $options_finded = array_filter(
//                $this->_options,
//                function ($e) use (&$attribute_id,&$attribute_key) {
//                    return $e["attribute_id"] == $attribute_id && $e["option_id"] == $attribute_key;
//                }
//            );

//            if ($attribute_code == 'performance_fabric') {
//                print_r($this->_options["254"]);exit;
//            }

        switch ($attribute->getFrontendInput()) {

                case 'multiselect':
                    break;
                default:
//                    $this->_item->attributes->{$attribute_code}->value = (count($options_finded)) ? $options_finded[$attribute_key]["value"] : $attribute_key;
                    $this->_item->attributes->{$attribute_code}->value =  (string)$this->_product->getAttributeText($attribute->getAttributeCode());
                    if (
                        isset($this->_product_attributes[$attribute->getAttributeCode()]["additional_data"]) &&
                        isset($this->_swatch_options[$this->_item->attributes->{$attribute_code}->key])
                    ) {
                        $additional_object = json_decode($this->_product_attributes[$attribute->getAttributeCode()]["additional_data"]);

                        if (property_exists($additional_object, "swatch_input_type")) {
                            $this->_item->attributes->{$attribute_code}->image = '/attribute/swatch' . $this->_swatch_options[$this->_item->attributes->{$attribute_code}->key]["value"];
                            $this->_item->attributes->{$attribute_code}->type = $additional_object->swatch_input_type;
                        }
                    }
                    break;

            }
    }

    protected function processAttributes()
    {
        $this->_item->attributes = new \stdClass();
        $this->_item->filters = new \stdClass();
        $this->_item->sortables = new \stdClass();

        $custom_attributes = $this->_product->getAttributes();

        foreach ($custom_attributes as $attribute) {
            $attribute_code = $attribute->getAttributeCode();

            // TODO manage multiselect attributes
            if (!$this->includeInIndex($attribute)) {
                continue;
            }

            $this->buildBaseAttributeData($attribute);
            if (in_array($attribute->getFrontendInput(), $this->allowedInputTypes, true)) {
                $this->buildInputAttributeData($attribute);
            } else {
                $this->_item->attributes->{$attribute_code}->value = $this->_product->getData($attribute->getAttributeCode());
            }

            if ((int) $attribute->getIsFilterable()) {
                $this->processFilters($this->_item->attributes->{$attribute_code});
            }

            if ((int) $attribute->getData('used_for_sort_by')) {
                $this->processSortables($this->_item->attributes->{$attribute_code});
            }
        }
    }

    protected function processFilters($attribute)
    {
        if ($attribute->value !== '' && $attribute->value !== null) {
            $attribute_code = $attribute->code;
            $this->_item->filters->{$attribute_code} = $attribute;
        }
    }

    protected function processSortables($attribute)
    {
        if ($attribute->value !== "" && $attribute->value !== null) {
            $attribute_code = $attribute->code;
            $this->_item->sortables->{$attribute_code} = $attribute;
        }
    }

    protected function processCategories()
    {
        $this->_item->categories = array_map('intval', $this->_product->getCategoryIds());
    }

    protected function processPrices()
    {
        $this->_item->pricing = new \stdClass();
        $this->_item->pricing->base_price = $this->_product->getPrice();
        $this->_item->pricing->final_price = $this->_product->getFinalPrice();

//        foreach ($this->_product->getPriceInfo()->getPrices() as $price) {
//            $this->_item->pricing->{$price->getPriceCode()} = $price->getValue();
//
//        }
    }

    protected function processInventory()
    {
        $this->_item->inventory = new \stdClass();
        $this->_item->inventory->in_stock = (bool)$this->_product->isSaleable();
        $this->_item->inventory->backorders = false;
        $this->_item->inventory->saleable = (bool)$this->_product->isSaleable();
    }
}
