<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Model\Indexer\Provider;

use FiloBlu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Select;
use Magento\Framework\Module\Manager;

/**
 * Class StockProvider
 * @package FiloBlu\Refilo\Model\Indexer\Provider
 */
class StockProvider implements QueryProviderInterface
{
    /** @var null */
    protected $ids;
    /** @var int */
    protected $website;
    /** @var ResourceConnection */
    private $resourceConnection;
    /** @var array */
    private $bindings = [];
    /**
     * @var Manager
     */
    private $moduleManager;

    /**
     * StockProvider constructor.
     * @param Manager $moduleManager
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        Manager $moduleManager,
        ResourceConnection $resourceConnection
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->moduleManager = $moduleManager;
    }

    /**
     * @return string
     */
    public function getQuery(): string
    {
        $select = $this->getSelectForDefaultStock();
        if ($this->ids !== null) {
            $select->where('product_id IN (' . implode(',', $this->ids) . ')');
        }

        if ($this->website !== null) {
            $select->where("sw.website_id = {$this->website}");
        }
        return $select->__toString();
    }

    /**
     * @return Select
     */
    protected function getSelectForDefaultStock()
    {
        return $this->resourceConnection->getConnection()
            ->select()
            ->from(
                [
                    'is' => 'inventory_stock_1'
                ],
                [
                    'stock_id' => 'stock_id',
                    '_id' => 'product_id',
                    'quantity' => 'quantity',
                    'is_salable' => 'is_salable',
                ]
            )->join(
                [
                    'issc' => 'inventory_stock_sales_channel'
                ],
                'is.stock_id = issc.stock_id',
                []
            )->join(
                ['sw' => 'store_website'],
                'sw.code = issc.code',
                ['website_id' => 'website_id']
            );
    }

    /**
     * @return array
     */
    public function getBindings(): array
    {
        return $this->bindings;
    }

    /**
     * @param array $ids
     * @return QueryProviderInterface
     */
    public function filter(array $ids): QueryProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    public function website(int $website_id): QueryProviderInterface
    {
        $this->website = $website_id;
        return $this;
    }

    public function store(int $store_id): QueryProviderInterface
    {
        return $this;
    }
}
